package ContactManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;

public class ContactManagerImpl implements ContactManager, Serializable {


	private static final long serialVersionUID = 869325199057541505L;
	private Set<PastMeeting> pastMeetings = new HashSet<>();
	private Set<FutureMeeting> futureMeetings = new HashSet<>();
	private Set<Contact> contactBook = new HashSet<>();
	private int contactCount;
	private final static String filename = "contacts.txt";
	private Calendar currentCal = Calendar.getInstance();
	private int GlobalMeetingCount;

	/**
	 * Static Factory Method (a la Joshua Bloch 'Effective Java 2nd Edition")
	 * 
	 * @return a contactManager either deserialized from contacts.txt or a new
	 *         manager, with fields initialized.
	 */


	public static ContactManagerImpl getInstance() {

		File file = new File(getFilename());
		ObjectInputStream in = null;
		if (file.exists()) {
			try {
				in = new ObjectInputStream(new FileInputStream("contacts.txt"));
				System.out.println("Read ContactManager from file");
				return (ContactManagerImpl) in.readObject();
			} catch (ClassCastException e) {
				System.err
						.println("Error casting object to a ContactManagerImpl");
			} catch (ClassNotFoundException e) {
				System.err.println("Class not found");
			} catch (IOException e) {
				System.err.println("Error reading object: " + e.getMessage());
			} finally {
				try {
					in.close();

				} catch (IOException e) {
					System.err.println(e.getMessage());
				}
			}

		} 
		System.out
				.println("Could not read ContactManager from file. Creating a New Manager.");
		return new ContactManagerImpl();


	}

	
	/**
	 * method to preserve singleton property a la Bloch (p 18). 
	 * @return the single instance of contactManagerImpl. 
	 */
	
	private Object readResolve(){
        return getInstance();
    }


	private ContactManagerImpl() {
		contactCount = 0;
		GlobalMeetingCount = 0;
	}

	@Override
	public int addFutureMeeting(Set<Contact> contacts, Calendar date) {

		Meeting newFutureMeeting = new FutureMeetingImpl(contacts, date);
		((MeetingImpl) newFutureMeeting).setId(GlobalMeetingCount);
		futureMeetings.add((FutureMeeting) newFutureMeeting);
		GlobalMeetingCount++;
		return newFutureMeeting.getId();

	}

	@Override
	public PastMeeting getPastMeeting(int id) {

		for (PastMeeting meetingToCheck : pastMeetings) {
			if (meetingToCheck.getId() == id) {
				return meetingToCheck;
			}
		}

		return null;
	}

	@Override
	public FutureMeeting getFutureMeeting(int id) {

		for (FutureMeeting meetingToCheck : futureMeetings){
	      if (meetingToCheck.getId() == id){
	    	  return meetingToCheck; 
	      }
	    }

	return null; 

	}



	@Override
	public Meeting getMeeting(int id) {

		if (getFutureMeeting(id) != null) {
			return getFutureMeeting(id);
		}

		else if (getPastMeeting(id) != null) {
			return getPastMeeting(id);
		}

		return null;
	}

	@Override
	public List<Meeting> getFutureMeetingList(Contact contact) {

		List<Meeting> meetingsWithContact;
		meetingsWithContact = new ArrayList<>();
		for (FutureMeeting toCheck : futureMeetings) {

			if (toCheck.getContacts().contains(contact)) {
				meetingsWithContact.add(toCheck);

			}
		}
		if (!meetingsWithContact.isEmpty()) {
			return meetingsWithContact;
		}
		return null;
	}

	@Override
	public List<Meeting> getFutureMeetingList(Calendar date) {
		List<Meeting> meetingsOnDate;
		meetingsOnDate = new ArrayList<>();
		for (FutureMeeting toCheck : futureMeetings) {

			if (toCheck.getDate().YEAR == (date.YEAR)
					&& toCheck.getDate().MONTH == date.MONTH
					&& toCheck.getDate().DATE == date.DATE) {
				meetingsOnDate.add(toCheck);

			}
		}
		if (!meetingsOnDate.isEmpty()) {
			return meetingsOnDate;
		}
		return null;
	}

	@Override
	public List<PastMeeting> getPastMeetingList(Contact contact) {
		List<PastMeeting> meetingsWithContact;
		meetingsWithContact = new ArrayList<>();

		for (PastMeeting toCheck : pastMeetings) {

			if (toCheck.getContacts().contains(contact)) {
				meetingsWithContact.add(toCheck);

			}
		}
		if (!meetingsWithContact.isEmpty()) {
			return meetingsWithContact;
		}
		return null;
	}

	@Override
	public void addNewPastMeeting(Set<Contact> contacts, Calendar date,
			String text) {

		PastMeeting newPastMeeting = new PastMeetingImpl(contacts, date, text);
		((MeetingImpl) newPastMeeting).setId(GlobalMeetingCount);
		GlobalMeetingCount++;
		pastMeetings.add(newPastMeeting);

	}

	@Override
	public void addMeetingNotes(int id, String text) {

		getMeeting(id).addNotes(text);

	}

	@Override
	public void addNewContact(String name, String notes) {
		ContactImpl newContact = new ContactImpl(name, contactCount);
		System.out.println("contact " + newContact.getName()
				+ " created with id " + newContact.getId());
		newContact.addNotes(notes);
		contactBook.add((Contact) newContact);
		contactCount++;

	}



/**
 * add a new contact with name and id
 * for testing purposes
 * @param name the name of the contact. 
 * @param id the id of the contact
 */

	public void addNewContact(String name, int id) {
		Contact newContact = new ContactImpl(name, id);
		System.out.println("contact " + newContact.getName()
				+ " created with id " + newContact.getId());

		contactBook.add((Contact) newContact);
		contactCount++;
	}
	
	@Override
	public Set<Contact> getContacts(int... ids) {

		Set<Contact> matchedContacts;
		matchedContacts = new HashSet<>();

		for (int id : ids){
		for (Contact toCheck : contactBook) {

				if (toCheck.equals(this.getContact(id))) {
					matchedContacts.add(toCheck);
					System.out.println("Found: " + toCheck.getName());
				}
			}
		}
		
		if (matchedContacts.isEmpty()) {
			System.out.println("Noting matched");
			return null;
		}
		return matchedContacts;
	}

	private Contact getContact(int id) {


		for (Contact toCheck : contactBook){
			if (toCheck.getId() == id){
				return toCheck; 
			}
		}
		return null;
	}

	@Override
	public Set<Contact> getContacts(String name) {

		Set<Contact> matchedContacts;
		matchedContacts = new HashSet<>();

		for (Contact toCheck : contactBook) {

			if (toCheck.getName().contains(name)) {
				matchedContacts.add(toCheck);

			}
		}
		if (!matchedContacts.isEmpty()) {
			return matchedContacts;
		}
		return null;
	}

	@Override
	public void flush() {
		FileOutputStream fos = null;
		ObjectOutputStream out = null;
		try {

			File file = new File(getFilename());
			if (!file.exists()) {
				file.createNewFile();

			}
			fos = new FileOutputStream(getFilename());
			out = new ObjectOutputStream(fos);
			out.writeObject(this);

			out.close();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * get a contact from a name
	 * @param name the name of the contact
	 * @return the contact object or null if it does not exist
	 */
	public Contact getContact(String name) {

		for (Contact toCheck : contactBook) {
			if (toCheck.getName().equals(name)) {
				return toCheck;
			}
		}
		return null;

	}


	public Meeting getFutureMeeting(Calendar date){


		for (FutureMeeting toCheck : futureMeetings){
			if (toCheck.getDate().equals(date)){
				return toCheck; 
			}
			}
		

		return null;
	}


	private int getGlobalMeetingCount(){
		return GlobalMeetingCount; 
	}

	public Set<Contact> getContactBook() {
		return contactBook;
	}

	public Set<FutureMeeting> getFutureMeetingSet() {
		return futureMeetings;
	}

	public Set<PastMeeting> getPastMeetingSet() {

		return pastMeetings; 


	}
	
	public static String getFilename() {
		return filename;
	}
	
	/**
	 * eclipse generated methods. 
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + GlobalMeetingCount;
		result = prime * result
				+ ((contactBook == null) ? 0 : contactBook.hashCode());
		result = prime * result + contactCount;
		result = prime * result
				+ ((currentCal == null) ? 0 : currentCal.hashCode());
		result = prime * result
				+ ((getFilename() == null) ? 0 : getFilename().hashCode());
		result = prime * result
				+ ((futureMeetings == null) ? 0 : futureMeetings.hashCode());
		result = prime * result
				+ ((pastMeetings == null) ? 0 : pastMeetings.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ContactManagerImpl other = (ContactManagerImpl) obj;
		if (GlobalMeetingCount != other.GlobalMeetingCount)
			return false;
		if (contactBook == null) {
			if (other.contactBook != null)
				return false;
		} else if (!contactBook.equals(other.contactBook))
			return false;
		if (contactCount != other.contactCount)
			return false;
		if (currentCal == null) {
			if (other.currentCal != null)
				return false;
		} else if (!currentCal.equals(other.currentCal))
			return false;
		if (getFilename() == null) {
			if (ContactManagerImpl.getFilename() != null)
				return false;
		} else if (!getFilename().equals(ContactManagerImpl.getFilename()))
			return false;
		if (futureMeetings == null) {
			if (other.futureMeetings != null)
				return false;
		} else if (!futureMeetings.equals(other.futureMeetings))
			return false;
		if (pastMeetings == null) {
			if (other.pastMeetings != null)
				return false;
		} else if (!pastMeetings.equals(other.pastMeetings))
			return false;
		return true;
	}



}
